@@ -25,19 +25,16 @@
 #include <gtk/gtk.h>
 
 #include "futaba.h"
-#include "futaba_ui.h"
+#include "futaba-ui.h"
 #include "action.h"
 #include "dirview.h"
 #include "imageviewer.h"
-#include "canvas.h"
 #include "dialog.h"
 #include "prefs.h"
 #include "bookmark.h"
 #include "history.h"
 #include "thumbnail.h"
 
-#define ZOOMIN 0.77
-#define ZOOMOUT 1.3
 /*
  *------------------------------------------------------
  *       Actions
@@ -74,27 +71,6 @@ static void cb_up_dir(GtkAction *action, gpointer data);
 static void cb_go_home(GtkAction *action, gpointer data);
 static void cb_refresh_dir(GtkAction *action, gpointer data);
 static void cb_change_column_type(GtkAction *action, gpointer data);
-static void cb_zoom_in(GtkAction *action, gpointer data);
-static void cb_zoom_out(GtkAction *action, gpointer data);
-static void cb_zoom_100(GtkAction *action, gpointer data);
-static void cb_fit_image(GtkAction *action, gpointer data);
-static void cb_right_rotation(GtkAction *action, gpointer data);
-static void cb_left_rotation(GtkAction *action, gpointer data);
-static void cb_mirror_rotation(GtkAction *action, gpointer data);
-static void cb_r_zoom_in(GtkAction *action, gpointer data);
-static void cb_r_zoom_out(GtkAction *action, gpointer data);
-static void cb_r_zoom_100(GtkAction *action, gpointer data);
-static void cb_r_fit_image(GtkAction *action, gpointer data);
-static void cb_r_right_rotation(GtkAction *action, gpointer data);
-static void cb_r_left_rotation(GtkAction *action, gpointer data);
-static void cb_r_mirror_rotation(GtkAction *action, gpointer data);
-static void cb_l_zoom_in(GtkAction *action, gpointer data);
-static void cb_l_zoom_out(GtkAction *action, gpointer data);
-static void cb_l_zoom_100(GtkAction *action, gpointer data);
-static void cb_l_fit_image(GtkAction *action, gpointer data);
-static void cb_l_right_rotation(GtkAction *action, gpointer data);
-static void cb_l_left_rotation(GtkAction *action, gpointer data);
-static void cb_l_mirror_rotation(GtkAction *action, gpointer data);
 static void cb_switch_file_display(GtkAction *action, gpointer data);
 static void cb_switch_dot_file(GtkAction *action, gpointer data);
 static void cb_switch_preview(GtkAction *action, gpointer data);
@@ -110,9 +86,6 @@ static void cb_show_col5(GtkAction *action, gpointer data);
 static void cb_show_coltitle(GtkAction *action, gpointer data);
 static void cb_set_dot_filter(GtkAction *action, gpointer data);
 static void cb_set_other_filter(GtkAction *action, gpointer data);
-static void cb_two_pages(GtkAction *action, gpointer data);
-static void cb_max_window(GtkAction *action, gpointer data);
-static void cb_fullscreen(GtkAction *action, gpointer data);
 static void cb_sort_mode(GtkRadioAction *action, GtkRadioAction *current, gpointer data);
 
 /* GtkActionEntry {
@@ -133,8 +106,6 @@ static GtkActionEntry actions[] = {
   {"Sort", NULL, N_("Sort")},
   {"Column", NULL, N_("Column")},
   {"Filter", NULL, N_("Filter")},
-  {"LeftImage", NULL, N_("left image")},
-  {"RightImage", NULL, N_("right image")},
 
   {"Open", "gtk-open", N_("Open"), NULL, N_("Open"), G_CALLBACK(cb_open)},
   {"OpenViewer", NULL, N_("Open Viewer"), "<control>V", N_("Open Image Viewer"), G_CALLBACK(cb_open_viewer)},
@@ -174,30 +145,6 @@ static GtkActionEntry actions[] = {
   {"ChangeColumnType", "fb-list", N_("fileinfo"), NULL, NULL, G_CALLBACK(cb_change_column_type)},
 
   {"SwitchPreview", "fb-preview", N_("preview"), "<alt>P", N_("Show/Hide Preview"), G_CALLBACK(cb_switch_preview)},
-
-  {"ZoomIn", "gtk-zoom-in", N_("zoom in"), "<release>J", NULL, G_CALLBACK(cb_zoom_in)},
-  {"ZoomOut", "gtk-zoom-out", N_("zoom out"), "<release>L", NULL, G_CALLBACK(cb_zoom_out)},
-  {"Zoom100", "gtk-zoom-100", N_("zoom 1:1"), "< release>O", NULL, G_CALLBACK(cb_zoom_100)},
-  {"ZoomFit", "gtk-zoom-fit", N_("fit image to window"),"<release>K", NULL, G_CALLBACK(cb_fit_image)},
-  {"RightRotation", NULL, N_("rigth rotation"), NULL, NULL, G_CALLBACK(cb_right_rotation)},
-  {"LeftRotation", NULL, N_("left rotation"), NULL, NULL, G_CALLBACK(cb_left_rotation)},
-  {"MirrorRotation", NULL, N_("mirror rotation"), NULL, NULL, G_CALLBACK(cb_mirror_rotation)},
-
-  {"R_ZoomIn", "gtk-zoom-in", N_("zoom in"), "<alt>J", NULL, G_CALLBACK(cb_r_zoom_in)},
-  {"R_ZoomOut", "gtk-zoom-out", N_("zoom out"), "<alt>L",  NULL, G_CALLBACK(cb_r_zoom_out)},
-  {"R_Zoom100", "gtk-zoom-100", N_("zoom 1:1"), "<alt>O", NULL, G_CALLBACK(cb_r_zoom_100)},
-  {"R_ZoomFit", "gtk-zoom-fit", N_("fit image to window"),"<alt>K", NULL, G_CALLBACK(cb_r_fit_image)},
-  {"R_RightRotation", NULL, N_("rigth rotation"), NULL, NULL, G_CALLBACK(cb_r_right_rotation)},
-  {"R_LeftRotation", NULL, N_("left rotation"), NULL, NULL, G_CALLBACK(cb_r_left_rotation)},
-  {"R_MirrorRotation", NULL, N_("mirror rotation"), NULL, NULL, G_CALLBACK(cb_r_mirror_rotation)},
-
-  {"L_ZoomIn", "gtk-zoom-in", N_("zoom in"), "<control>J", NULL, G_CALLBACK(cb_l_zoom_in)},
-  {"L_ZoomOut", "gtk-zoom-out", N_("zoom out"), "<control>L",  NULL, G_CALLBACK(cb_l_zoom_out)},
-  {"L_Zoom100", "gtk-zoom-100", N_("zoom 1:1"), "<control>O", NULL, G_CALLBACK(cb_l_zoom_100)},
-  {"L_ZoomFit", "gtk-zoom-fit", N_("fit image to window"),"<control>K", NULL, G_CALLBACK(cb_l_fit_image)},
-  {"L_RightRotation", NULL, N_("rigth rotation"), NULL, NULL, G_CALLBACK(cb_l_right_rotation)},
-  {"L_LeftRotation", NULL, N_("left rotation"), NULL, NULL, G_CALLBACK(cb_l_left_rotation)},
-  {"L_MirrorRotation", NULL, N_("mirror rotation"), NULL, NULL, G_CALLBACK(cb_l_mirror_rotation)},
 };
 static guint actions_len = G_N_ELEMENTS(actions);
 
@@ -217,10 +164,6 @@ static GtkToggleActionEntry toggle_actions[] = {
 
   {"FilterDot", NULL, N_("Dot file"), NULL, NULL, G_CALLBACK(cb_set_dot_filter), TRUE},
   {"FilterOther", NULL, N_("Other file"), NULL, NULL, G_CALLBACK(cb_set_other_filter), TRUE},
-
-  {"TwoWindow", NULL, N_("two pages display"), "<release>space", NULL, G_CALLBACK(cb_two_pages), FALSE},
-  {"MaxWindow", NULL, N_("Max Window"), "<release>M", NULL, G_CALLBACK(cb_max_window), FALSE},
-  {"FullScreen", NULL, N_("full screen"), "<release>F", NULL, G_CALLBACK(cb_fullscreen), FALSE},
 };
 static guint toggle_actions_len = G_N_ELEMENTS(toggle_actions);
 
@@ -243,10 +186,10 @@ cb_show_file_menu(GtkAction *action,
 	fb_ui_set_sensitive(futaba->ui_manager, "/MenuBar/FileMenu/OpenViewer", flag);
 	fb_ui_set_sensitive(futaba->ui_manager, "/MenuBar/FileMenu/OpenThumb", flag);
 
-	flag = (futaba->viewer != NULL);
+	flag = (focus_viewer != NULL);
 	fb_ui_set_sensitive(futaba->ui_manager, "/MenuBar/FileMenu/CloseViewer", flag);
 
-	flag = (futaba->thumbwin != NULL);
+	flag = (focus_thumbwin != NULL);
 	fb_ui_set_sensitive(futaba->ui_manager, "/MenuBar/FileMenu/CloseThumb", flag);
 }
 
@@ -330,6 +273,7 @@ static void
 cb_open_viewer(GtkAction *action,
 	       gpointer data)
 {
+	GList *path_list;
 	Futaba *futaba = data;
 
 	if (!futaba->image_num) {
@@ -338,22 +282,28 @@ cb_open_viewer(GtkAction *action,
 		return;
 	}
 
-	fb_imageviewer_start(futaba);
+	path_list = fb_dirview_get_path_list(futaba, IMAGE);
+	fb_imageviewer_start(path_list, 0);
+
+	path_list_free(path_list);
 }
 
 static void
 cb_open_thumb(GtkAction *action,
 	      gpointer data)
 {
+	GList *path_list;
 	Futaba *futaba = data;
 
 	if (!futaba->image_num) {
-		fb_statusbar_push(_("No image !!"));
+		fb_statusbar_push (_("No image !!"));
 
 		return;
 	}
 	
-	fb_thumbwin_start(futaba);
+	path_list = fb_dirview_get_path_list (futaba, IMAGE);
+	fb_thumbwin_start (path_list);
+	path_list_free (path_list);
 }
 
 static void
@@ -362,9 +312,9 @@ cb_close_viewer(GtkAction *action,
 {
 	Futaba *futaba = data;
 
-	if (!futaba->viewer) return;
+	if (!focus_viewer) return;
 
-	g_signal_emit_by_name(G_OBJECT(futaba->viewer->top), "destroy", futaba);
+	g_signal_emit_by_name(G_OBJECT(focus_viewer->top), "destroy", focus_viewer);
 }
 
 static void
@@ -373,9 +323,9 @@ cb_close_thumb(GtkAction *action,
 {
 	Futaba *futaba = data;
 
-	if (!futaba->thumbwin) return;
+	if (!focus_thumbwin) return;
 
-	g_signal_emit_by_name(G_OBJECT(futaba->thumbwin->top), "destroy", futaba);
+	g_signal_emit_by_name(G_OBJECT(focus_thumbwin->top), "destroy", focus_thumbwin);
 }
 
 static void
@@ -711,355 +661,6 @@ cb_switch_preview(GtkAction *action,
 	fb_dirview_set_preview(futaba, !futaba->show_preview);
 }
 
-static void
-cb_zoom_in(GtkAction *action,
-	   gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	fb_imagecanvas_zooming(futaba->viewer->first, futaba->viewer->first->scale * ZOOMIN);
-
-	if (futaba->viewer->two_pages)
-		fb_imagecanvas_zooming(futaba->viewer->second, futaba->viewer->second->scale * ZOOMIN);
-}
-
-static void
-cb_zoom_out(GtkAction *action,
-	    gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	fb_imagecanvas_zooming(futaba->viewer->first, futaba->viewer->first->scale * ZOOMOUT);
-
-	if (futaba->viewer->two_pages)
-		fb_imagecanvas_zooming(futaba->viewer->second, futaba->viewer->second->scale * ZOOMOUT);
-}
-
-static void
-cb_zoom_100(GtkAction *action,
-	    gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	fb_imagecanvas_zooming(futaba->viewer->first, 1.0);
-
-	if (futaba->viewer->two_pages)
-		fb_imagecanvas_zooming(futaba->viewer->second, 1.0);
-}
-
-static void
-cb_fit_image(GtkAction *action,
-	     gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	fb_imagecanvas_zoomfit(futaba->viewer->first);
-
-	if (futaba->viewer->two_pages)
-		fb_imagecanvas_zoomfit(futaba->viewer->second);
-}
-
-static void
-cb_right_rotation(GtkAction *action,
-		  gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	fb_imagecanvas_rotate(futaba->viewer->first, FALSE);
-
-	if (futaba->viewer->two_pages)
-		fb_imagecanvas_rotate(futaba->viewer->second, FALSE);
-}
-
-static void
-cb_left_rotation(GtkAction *action,
-		 gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	fb_imagecanvas_rotate(futaba->viewer->first, TRUE);
-
-	if (futaba->viewer->two_pages)
-		fb_imagecanvas_rotate(futaba->viewer->second, TRUE);
-}
-
-static void
-cb_mirror_rotation(GtkAction *action,
-		   gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	fb_imagecanvas_mirror(futaba->viewer->first, TRUE, FALSE);
-
-	if (futaba->viewer->two_pages)
-		fb_imagecanvas_mirror(futaba->viewer->second, TRUE, FALSE);
-}
-
-
-static void
-cb_r_zoom_in(GtkAction *action,
-	     gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zooming(futaba->viewer->first, futaba->viewer->first->scale * ZOOMIN);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zooming(futaba->viewer->second,  futaba->viewer->second->scale * ZOOMIN);
-}
-
-static void
-cb_r_zoom_out(GtkAction *action,
-	      gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zooming(futaba->viewer->first, futaba->viewer->first->scale * ZOOMOUT);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zooming(futaba->viewer->second, futaba->viewer->second->scale * ZOOMOUT);
-}
-
-static void
-cb_r_zoom_100(GtkAction *action,
-	      gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zooming(futaba->viewer->first, 1.0);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zooming(futaba->viewer->second, 1.0);
-}
-
-static void
-cb_r_fit_image(GtkAction *action,
-	       gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zoomfit(futaba->viewer->first);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zoomfit(futaba->viewer->second);
-}
-
-static void
-cb_r_right_rotation(GtkAction *action,
-		    gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_rotate(futaba->viewer->first, FALSE);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_rotate(futaba->viewer->second, FALSE);
-}
-
-static void
-cb_r_left_rotation(GtkAction *action,
-		   gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_rotate(futaba->viewer->first, TRUE);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_rotate(futaba->viewer->second, TRUE);
-}
-
-static void
-cb_r_mirror_rotation(GtkAction *action,
-		     gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_mirror(futaba->viewer->first, TRUE, FALSE);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_mirror(futaba->viewer->second, TRUE, FALSE);
-}
-
-
-static void
-cb_l_zoom_in(GtkAction *action,
-	     gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zooming(futaba->viewer->second, futaba->viewer->second->scale * ZOOMIN);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zooming(futaba->viewer->first, futaba->viewer->first->scale * ZOOMIN);
-}
-
-static void
-cb_l_zoom_out(GtkAction *action,
-	      gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zooming(futaba->viewer->second, futaba->viewer->second->scale * ZOOMOUT);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zooming(futaba->viewer->first, futaba->viewer->first->scale * ZOOMOUT);
-}
-
-static void
-cb_l_zoom_100(GtkAction *action,
-	      gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zooming(futaba->viewer->second, 1.0);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zooming(futaba->viewer->first, 1.0);
-}
-
-static void
-cb_l_fit_image(GtkAction *action,
-	       gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_zoomfit(futaba->viewer->second);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_zoomfit(futaba->viewer->first);
-}
-
-static void
-cb_l_right_rotation(GtkAction *action,
-		    gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_rotate(futaba->viewer->second, FALSE);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_rotate(futaba->viewer->first, FALSE);
-}
-
-static void
-cb_l_left_rotation(GtkAction *action,
-		   gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_rotate(futaba->viewer->second, TRUE);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_rotate(futaba->viewer->first, TRUE);
-}
-
-static void
-cb_l_mirror_rotation(GtkAction *action,
-		     gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->open_way == RIGHT)
-		fb_imagecanvas_mirror(futaba->viewer->second, TRUE, FALSE);
-	else if (futaba->viewer->open_way == LEFT)
-		fb_imagecanvas_mirror(futaba->viewer->first, TRUE, FALSE);
-}
-
-static void
-cb_two_pages(GtkAction *action,
-	     gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-
-	if (futaba->viewer->two_pages)
-		fb_imageviewer_fixup_window(futaba->viewer);
-	else
-		fb_imageviewer_split_window(futaba->viewer);
-}
-
-static void
-cb_max_window(GtkAction *action,
-	      gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-	if (futaba->viewer->size > WINDOW_FULL) return;
-
-	if (futaba->viewer->size > WINDOW_MAX) {
-		gtk_window_unmaximize(GTK_WINDOW(futaba->viewer->top));
-		futaba->viewer->size ^= WINDOW_MAX;
-	}
-	else {
-		gtk_window_maximize(GTK_WINDOW(futaba->viewer->top));
-		gtk_window_move(GTK_WINDOW(futaba->viewer->top), 0, 0);
-		futaba->viewer->size ^= WINDOW_MAX;
-	}
-}
-
-static void
-cb_fullscreen(GtkAction *action,
-	      gpointer data)
-{
-	Futaba *futaba = data;
-
-	if (!futaba->viewer) return;
-	/*   if (!futaba->viewer->focus) return; */
-
-	if (futaba->viewer->size > WINDOW_FULL) {
-		gtk_window_unfullscreen(GTK_WINDOW(futaba->viewer->top));
-		futaba->viewer->size ^= WINDOW_FULL;
-	}
-	else {
-		gtk_window_fullscreen(GTK_WINDOW(futaba->viewer->top));
-		futaba->viewer->size ^= WINDOW_FULL;
-	}
-}
-
 static void
 cb_show_tbar(GtkAction *action,
 	     gpointer data)
