@@ -22,11 +22,13 @@
 #  include "config.h"
 #endif
 
+#include <unistd.h>
+
 #include <gtk/gtk.h>
 
 #include "futaba.h"
-#include "futaba_ui.h"
-#include "fileutil.h"
+#include "futaba-ui.h"
+#include "file-utils.h"
 #include "dirview.h"
 #include "dialog.h"
 
@@ -504,10 +506,10 @@ fb_rm_file(gchar *path)
 {
 	GtkWidget *dialog;
 
-	if (!path) return FALSE;
-	if (!g_file_test(path, G_FILE_TEST_EXISTS)) return FALSE;
+	if (! path || ! *path) return FALSE;
+	if (!path_is_exists(path)) return FALSE;
 
-	if (!delete_file(path)) {
+	if (unlink (path)) {
 		dialog = gtk_message_dialog_new(NULL,
 						GTK_DIALOG_MODAL,
 						GTK_MESSAGE_ERROR,
@@ -532,16 +534,16 @@ fb_rm_dir(gchar *path)
 	GList *first, *entries;
 	GtkWidget *prog_dlg, *dialog;
 
-	if (!path) return FALSE;
-	if (!g_file_test(path, G_FILE_TEST_EXISTS)) return FALSE;
+	if (! path || ! *path) return FALSE;
+	if (!path_is_dir(path)) return FALSE;
 
-	first = get_dir_entries(path, FALSE, TRUE);
+	first = get_path_list(path, TRUE);
 
 	length = g_list_length(first);
 	if (length == 1) {
-		free_dir_entries(first);
+		path_list_free(first);
 
-		if (!delete_dir(path)) {
+		if (rmdir (path)) {
 			dialog = gtk_message_dialog_new(NULL,
 							GTK_DIALOG_MODAL,
 							GTK_MESSAGE_ERROR,
@@ -566,7 +568,7 @@ fb_rm_dir(gchar *path)
 	gtk_widget_destroy(dialog);
 
 	if (result == GTK_RESPONSE_CANCEL) {
-		free_dir_entries(first);
+		path_list_free(first);
       
 		return FALSE;
 	}
@@ -581,18 +583,18 @@ fb_rm_dir(gchar *path)
 
 		if (FB_PROGRESS_DIALOG(prog_dlg)->interrupt) {
 			gtk_widget_destroy(prog_dlg);
-			free_dir_entries(first);
+			path_list_free(first);
 
 			return FALSE;
 		}
 
 		entry = entries->data;
-		if (g_file_test(entry, G_FILE_TEST_IS_DIR))
-			is_delete = delete_dir(entry);
+		if (path_is_dir( entry))
+			is_delete = rmdir (entry);
 		else
-			is_delete = delete_file(entry);
+			is_delete = unlink (entry);
 
-		if (!is_delete && (result != GTK_RESPONSE_YES)) {
+		if (is_delete && (result != GTK_RESPONSE_YES)) {
 			dialog = gtk_message_dialog_new(NULL,
 							GTK_DIALOG_MODAL,
 							GTK_MESSAGE_QUESTION,
@@ -603,7 +605,7 @@ fb_rm_dir(gchar *path)
 
 			if (result == GTK_RESPONSE_NO) {
 				gtk_widget_destroy(prog_dlg);
-				free_dir_entries(first);
+				path_list_free(first);
 
 				return FALSE;
 			}
@@ -615,7 +617,7 @@ fb_rm_dir(gchar *path)
 						fraction, entry);
 	}
 
-	free_dir_entries(first);
+	path_list_free(first);
 	gtk_widget_destroy(prog_dlg);
 
 	return TRUE;
@@ -628,7 +630,7 @@ fb_rmdir_start(gchar *path)
 	GtkWidget *dialog;
 
 	if (!path) return FALSE;
-	if (!g_file_test(path, G_FILE_TEST_EXISTS)) return FALSE;
+	if (!path_is_exists(path)) return FALSE;
 
 	dialog = gtk_message_dialog_new(NULL,
 					GTK_DIALOG_MODAL,
@@ -641,7 +643,7 @@ fb_rmdir_start(gchar *path)
 	if (result == GTK_RESPONSE_CANCEL) return FALSE;
 
 	/* delete file */
-	if (g_file_test(path, G_FILE_TEST_IS_DIR))
+	if (path_is_dir(path))
 		result = fb_rm_dir(path);
 	else
 		result = fb_rm_file(path);
@@ -664,7 +666,7 @@ fb_paste_file(gchar *src_path,
 
 	if (!src_path || !dest_path) return FALSE;
 
-	if (g_file_test(dest_path, G_FILE_TEST_EXISTS)) {
+	if (path_is_exists(dest_path)) {
 		dialog = gtk_message_dialog_new(NULL,
 						GTK_DIALOG_MODAL,
 						GTK_MESSAGE_QUESTION,
@@ -676,7 +678,7 @@ fb_paste_file(gchar *src_path,
 		if (is_rewrite == GTK_RESPONSE_CANCEL) return FALSE;
 	}
 
-	if (!copy_file(src_path, dest_path)) {
+	if (!file_copy(src_path, dest_path)) {
 		dialog = gtk_message_dialog_new(NULL,
 						GTK_DIALOG_MODAL,
 						GTK_MESSAGE_ERROR,
@@ -690,7 +692,7 @@ fb_paste_file(gchar *src_path,
 	}
 
 	if (!is_copy) {
-		if (!delete_file(src_path)) {
+		if (unlink(src_path)) {
 			dialog = gtk_message_dialog_new(NULL,
 							GTK_DIALOG_MODAL,
 							GTK_MESSAGE_ERROR,
@@ -753,9 +755,10 @@ fb_paste_dir(gchar *src_path,
 	GList *first, *entries;
 	GtkWidget *prog_dlg, *dialog;
 
-	if (!src_path || !dest_path) return FALSE;
+	if (! src_path || ! *src_path) return FALSE;
+	if (! dest_path || ! *dest_path) return FALSE;
 
-	first = get_dir_entries(src_path, FALSE, TRUE);
+	first = get_path_list(src_path, TRUE);
 	first = g_list_reverse(first);
 
 	pos = 0;
@@ -770,7 +773,7 @@ fb_paste_dir(gchar *src_path,
 
 		if (FB_PROGRESS_DIALOG(prog_dlg)->interrupt) {
 			gtk_widget_destroy(prog_dlg);
-			free_dir_entries(first);
+			path_list_free(first);
 
 			return FALSE;
 		}
@@ -781,8 +784,7 @@ fb_paste_dir(gchar *src_path,
 		if (!dest_file) continue;
 
 		/* check destination */
-		if ((response != GTK_RESPONSE_APPLY) &&
-		    g_file_test(dest_file, G_FILE_TEST_EXISTS)) {
+		if ((response != GTK_RESPONSE_APPLY) && path_is_exists (dest_file)) {
 			dialog = gtk_message_dialog_new(NULL,
 							GTK_DIALOG_MODAL,
 							GTK_MESSAGE_QUESTION,
@@ -798,17 +800,17 @@ fb_paste_dir(gchar *src_path,
 
 			if (response == GTK_RESPONSE_NO) {
 				gtk_widget_destroy(prog_dlg);
-				free_dir_entries(first);
+				path_list_free(first);
 				g_free(dest_file);
 	     
 				return FALSE;
 			}
 		}
 
-		if (g_file_test(src_file, G_FILE_TEST_IS_DIR))
-			result = MIN(result, copy_dir(src_file, dest_file));
+		if (path_is_dir(src_file))
+			result = MIN(result, dir_copy(src_file, dest_file));
 		else
-			result = MIN(result, copy_file(src_file, dest_file));
+			result = MIN(result, file_copy(src_file, dest_file));
 		g_free(dest_file);
 
 		pos++;
@@ -840,10 +842,10 @@ fb_paste_dir(gchar *src_path,
 		for (entries = g_list_reverse(first); entries; entries = entries->next) {
 			while (gtk_events_pending()) gtk_main_iteration();
 
-			if (g_file_test(entries->data, G_FILE_TEST_IS_DIR))
-				is_delete = MIN(is_delete, delete_dir(entries->data));
+			if (path_is_dir( entries->data))
+				is_delete = MIN(is_delete, rmdir(entries->data));
 			else
-				is_delete = MIN(is_delete, delete_file(entries->data));
+				is_delete = MIN(is_delete, unlink(entries->data));
 
 			pos++;
 			fraction = (gfloat) pos / (gfloat) length;
@@ -851,7 +853,7 @@ fb_paste_dir(gchar *src_path,
 							fraction, entries->data);
 		}
 
-		if (!is_delete) {
+		if (is_delete) {
 			dialog = gtk_message_dialog_new(NULL,
 							GTK_DIALOG_MODAL,
 							GTK_MESSAGE_ERROR,
@@ -863,7 +865,7 @@ fb_paste_dir(gchar *src_path,
 		}
 	}
   
-	free_dir_entries(first);
+	path_list_free(first);
 	gtk_widget_destroy(prog_dlg);
 
 	return TRUE;
@@ -876,9 +878,9 @@ fb_paste_start(Futaba *futaba,
 	gboolean result;
 
 	if (!futaba->paste_path) return FALSE;
-	if (!dest_path) return FALSE;
+	if (! dest_path || ! *dest_path) return FALSE;
 
-	if (g_file_test(futaba->paste_path, G_FILE_TEST_IS_DIR))
+	if (path_is_dir(futaba->paste_path))
 		result = fb_paste_dir(futaba->paste_path, dest_path, futaba->is_copy);
 	else
 		result = fb_paste_file(futaba->paste_path, dest_path, futaba->is_copy);
