@@ -32,9 +32,11 @@
 #include <gtk/gtk.h>
 
 #include "futaba.h"
-#include "futaba_ui.h"
+#include "futaba-ui.h"
 #include "dirview.h"
-#include "fileutil.h"
+#include "file-utils.h"
+#include "fr-archive.h"
+#include "dialog.h"
 
 gboolean cdir_filter;
 gboolean pdir_filter;
@@ -57,212 +59,19 @@ static const gchar *week[7] = {
 	N_("Sat")
 };
 
-static const gchar *ani_extensions[] = {
-	"ani",
-};
-static const gchar *bmp_extensions[] = {
-	"bmp",
-};
-static const gchar *gif_extensions[] = {
-	"gif",
-};
-static const gchar *icon_extensions[] = {
-	"ico",
-	"cur",
-};
-static const gchar *jpeg_extensions[] = {
-	"jpg",
-	"jpeg",
-	"jpe",
-};
-static const gchar *png_extensions[] = {
-	"png",
-};
-static const gchar *ppm_extensions[] = {
-	"ppm",
-};
-static const gchar *pgm_extensions[] = {
-	"pgm",
-};
-static const gchar *pbm_extensions[] = {
-	"pbm",
-};
-static const gchar *pnm_extensions[] = {
-	"pnm",
-	"pbm",
-	"pgm",
-	"ppm",
-};
-static const gchar *ras_extensions[] = {
-	"ras",
-};
-static const gchar *tga_extensions[] = {
-	"tga",
-	"targa",
-};
-static const gchar *tiff_extensions[] = {
-	"tiff",
-	"tif",
-};
-static const gchar *wbmp_extensions[] = {
-	"wbmp",
-};
-static const gchar *xbm_extensions[] = {
-	"xbm",
-};
-static const gchar *xpm_extensions[] = {
-	"xpm",
-};
-
-static MimeTypeEntry mime_types[] = {
-   {
-      mime_type:      "application/x-navi-animation",
-      description:    N_("The ANI image format"),
-      extensions:     ani_extensions,
-      extensions_len: sizeof (ani_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/bmp",
-      description:    N_("The BMP image format"),
-      extensions:     bmp_extensions,
-      extensions_len: sizeof (bmp_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-bmp",
-      description:    N_("The BMP image format"),
-      extensions:     bmp_extensions,
-      extensions_len: sizeof (bmp_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-MS-bmp",
-      description:    N_("The BMP image format"),
-      extensions:     bmp_extensions,
-      extensions_len: sizeof (bmp_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/gif",
-      description:    N_("The GIF image format"),
-      extensions:     gif_extensions,
-      extensions_len: sizeof (gif_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-icon",
-      description:    N_("The ICO image format"),
-      extensions:     icon_extensions,
-      extensions_len: sizeof (icon_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/jpeg",
-      description:    N_("The JPEG image format"),
-      extensions:     jpeg_extensions,
-      extensions_len: sizeof (jpeg_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/png",
-      description:    N_("The PNG image format"),
-      extensions:     png_extensions,
-      extensions_len: sizeof (png_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-portable-anymap",
-      description:    N_("Portable Any Map Image"),
-      extensions:     pnm_extensions,
-      extensions_len: sizeof (pnm_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-portable-bitmap",
-      description:    N_("The PBM image format"),
-      extensions:     pbm_extensions,
-      extensions_len: sizeof (pbm_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-portable-graymap",
-      description:    N_("The PGM image format"),
-      extensions:     pgm_extensions,
-      extensions_len: sizeof (pgm_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-portable-pixmap",
-      description:    N_("The PPM image format"),
-      extensions:     ppm_extensions,
-      extensions_len: sizeof (ppm_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-cmu-raster",
-      description:    N_("The Sun raster image format"),
-      extensions:     ras_extensions,
-      extensions_len: sizeof (ras_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-sun-raster",
-      description:    N_("The Sun raster image format"),
-      extensions:     ras_extensions,
-      extensions_len: sizeof (ras_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-tga",
-      description:    N_("The Targa image format"),
-      extensions:     tga_extensions,
-      extensions_len: sizeof (tga_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/tiff",
-      description:    N_("The TIFF image format"),
-      extensions:     tiff_extensions,
-      extensions_len: sizeof (tiff_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/vnd.wap.wbmp",
-      description:    N_("The WBMP image format"),
-      extensions:     wbmp_extensions,
-      extensions_len: sizeof (wbmp_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-xbitmap",
-      description:    N_("The XBM image format"),
-      extensions:     xbm_extensions,
-      extensions_len: sizeof (xbm_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-   {
-      mime_type:      "image/x-xpixmap",
-      description:    N_("The XPM image format"),
-      extensions:     xpm_extensions,
-      extensions_len: sizeof (xpm_extensions) / sizeof (gchar *),
-      icon:           NULL,
-   },
-};
-
 /*
  *-------------------------------------
  *     string utility
  *-------------------------------------
  */
-static gchar *get_absolute_path(gchar *path);
+static gchar *get_absolute_path(const gchar *path);
 static gchar *str_mode2str(mode_t mode);
 static gchar *str_uid2str(uid_t uid);
 static gchar *str_gid2str(gid_t gid);
 static gchar *str_size2str(off_t size);
 static gchar *str_entries2str(gint count);
 static gchar *str_mtime2str(time_t mtime);
-static gint count_dir_entries(gchar *path, gboolean dot_filter, gboolean other_filter);
+static gint count_dir_entries(const gchar *path, gboolean dot_filter, gboolean other_filter);
 
 gchar *
 str_local_to_utf8(gchar *string)
@@ -303,36 +112,35 @@ str_utf8_to_local(gchar *string)
 }
 
 static gchar *
-get_absolute_path(gchar *path)
+get_absolute_path(const gchar *path)
 {
-	gchar *base, *tmp, *abs;
+	gint p;
+	gchar *new_path;
+	const gchar *ptr = path;
 
-	if (path[0] != '/') return NULL;
+	if (ptr[0] != '/') return NULL;
 
-	tmp = g_strdup(path);
-	if (tmp[0] == '/' && tmp[1] =='/') {
-		tmp[0] = ' ';
-		tmp = g_strchug(tmp);
+	/* remove ".", ".." from path */
+	p = strlen (path);
+	if (ptr[p] == '.') {
+		p--;
+		if (ptr[p] == '/') p--;
+
+		if (ptr[p] == '.') {
+			p -= 2;
+			while ((ptr[p] != '/') && (p > 0)) p--;
+		}
 	}
+	new_path = g_strndup (path, (guint)p);
 
-	base = g_path_get_basename(tmp);
-	if (strcmp(base, ".") == 0)
-		abs = g_path_get_dirname(tmp);
-	else if (strcmp(base, "..") == 0) {
-		gchar *tmp1;
-
-		tmp1 = g_path_get_dirname(tmp);
-		abs = g_path_get_dirname(tmp1);
-		g_free(tmp1);
-	}
-	else {
-		abs = g_strdup(tmp);
+	p = 1;
+	while (ptr[p] == '/') {
+		new_path[p - 1] = ' ';
+		p++;
 	}
+	if (new_path[0] == ' ') new_path = g_strchug (new_path);
 
-	g_free(base);
-	g_free(tmp);
-
-	return abs;
+	return new_path;
 }
 
 static gchar *
@@ -441,70 +249,29 @@ str_mtime2str(time_t mtime)
 	return NULL;
 }
 
-
-gboolean
-is_archive(gchar *name)
-{
-	if (!name) return FALSE;
-
-	if (g_str_has_suffix(name, ".tar") ||
-	    g_str_has_suffix(name, ".bz2") ||
-	    g_str_has_suffix(name, ".gz")  ||
-	    g_str_has_suffix(name, ".zip") ||
-	    g_str_has_suffix(name, ".ZIP") ||
-	    g_str_has_suffix(name, ".lha") ||
-	    g_str_has_suffix(name, ".rar"))
-		{
-			return TRUE;
-		}
-
-	return FALSE;
-}
- 
-gboolean
-is_imagefile(gchar *name)
-{
-	if (!name) return FALSE;
-
-	if (g_str_has_suffix(name, ".jpeg") ||
-	    g_str_has_suffix(name, ".jpg")  ||
-	    g_str_has_suffix(name, ".JPG")  ||
-	    g_str_has_suffix(name, ".gif")  ||
-	    g_str_has_suffix(name, ".png")  || 
-	    g_str_has_suffix(name, ".tiff") ||
-	    g_str_has_suffix(name, ".xpm")  ||
-	    g_str_has_suffix(name, ".bmp")  ||
-	    g_str_has_suffix(name, ".xbm"))
-		{
-			return TRUE;
-		}
-
-	return FALSE;
-}
-
 static gint
-count_dir_entries(gchar *path,
+count_dir_entries(const gchar *path,
 		  gboolean dot_filter,
 		  gboolean other_filter)
 {
 	gint count = 0;
-	GList *entries, *node;
+	GList *path_list, *node;
 
-	if (!path) return -1;
+	if (! path || ! *path) return -1;
 
-	entries = get_dir_entries(path, dot_filter, FALSE);
-	for (node = entries; node; node = node->next) {
+	path_list = get_path_list(path, FALSE);
+	for (node = path_list; node; node = node->next) {
 		if (!node->data) continue;
 
 		if (other_filter &&
-		    !g_file_test(node->data, G_FILE_TEST_IS_DIR) &&
-		    !(is_imagefile(node->data) || is_archive(node->data)))
+		    !path_is_dir(node->data) &&
+		    !(file_is_image(node->data) || file_is_archive(node->data)))
 			continue;
 
 		count++;
 	}
 
-	free_dir_entries(entries);
+	path_list_free(path_list);
 
 	return count;
 }
@@ -822,17 +589,22 @@ dirlist_append(GtkListStore *list,
 	gchar *permission, *owner, *group, *size, *mtime;
 	GtkTreeIter iter;
 
-	if (!path) return iter;
+	if (! path || ! *path) return iter;
 	if (!GTK_IS_LIST_STORE(list)) return iter;
 
+	name = g_path_get_basename(path);
+	if (dot_filter && name[0] == '.') {
+		g_free(name);
+		return iter;
+	}
+
 	if (stat(path, &file)) return iter;
 
-	name = g_path_get_basename(path);
 	if (S_ISDIR(file.st_mode))
 		type = DIRECTORY;
-	else if (S_ISREG(file.st_mode) && is_imagefile(name))
+	else if (S_ISREG(file.st_mode) && file_is_image(name))
 		type = IMAGE;
-	else if (S_ISREG(file.st_mode) && is_archive(name))
+	else if (S_ISREG(file.st_mode) && file_is_archive(name))
 		type = ARCHIVE;
 	else
 		type = OTHER;
@@ -845,7 +617,7 @@ dirlist_append(GtkListStore *list,
 	utf8_name = str_local_to_utf8(name);
 	g_free(name);
 
-	readable = is_readable(path);
+	readable = path_is_readable(path);
 
 	if (type == DIRECTORY)
 		size = str_entries2str(count_dir_entries(path, dot_filter, other_filter));
@@ -926,10 +698,10 @@ dirlist_set_dir(GtkListStore *list,
 
 	gtk_list_store_clear(list);
 
-	entries = get_dir_entries(path, dot_filter, FALSE);
+	entries = get_path_list(path, FALSE);
 	for (i = 0; entry = g_list_nth_data(entries, i); i++)
 		dirlist_append(list, entry, dot_filter, other_filter);
-	free_dir_entries(entries);
+	path_list_free(entries);
 
 	if (!pdir_filter) dirlist_add_pdir(list, path, dot_filter, other_filter);
 	if (!cdir_filter) dirlist_add_cdir(list, path, dot_filter, other_filter);
@@ -1451,44 +1223,17 @@ fb_dirview_new(Futaba *futaba)
 	return hpaned;
 }
 
-
-gboolean
-fb_dirview_set_history(Futaba *futaba,
-		       gchar *path)
+static void
+fb_dirview_show_message(Futaba *futaba,
+			gchar *path)
 {
 	gint num;
-	gchar *abs_path, *utf8, *msg_num, *msg_path, *message;
-	GtkTreeModel *filter;
-	GtkAdjustment *adjust;
+	gchar *utf8, *msg_num, *msg_path, *message;
 
-	if (!path) return FALSE;
-	if (!futaba->treeview) return FALSE;
-	if (!futaba->liststore) return FALSE;
+	if (! path || ! *path) return;
 
-	if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
-		fb_statusbar_push(_("Directory don't exist."));
+	fb_locationbar_set_text(futaba, path);
 
-		return FALSE;
-	}
-
-	abs_path = get_absolute_path(path);
-	/* set dirview */
-	if (!dirlist_set_dir(futaba->liststore, abs_path, futaba->dot_filter, futaba->other_filter)) {
-		g_free(abs_path);
-		return FALSE;
-	}
-
-	futaba->workdir = g_strdup(abs_path);
-	futaba->writable = is_writable(abs_path);
-	futaba->image_num = dirlist_get_count(futaba->liststore, IMAGE);
-
-	/* set row position */
-	adjust = gtk_tree_view_get_vadjustment(GTK_TREE_VIEW(futaba->treeview));
-	gtk_adjustment_set_value(adjust, 0.0);
-
-	fb_locationbar_set_text(futaba, abs_path);
-
-	/* set statusbar */
 	num = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(futaba->liststore), NULL);
 	if (!pdir_filter) num--;
 	if (!cdir_filter) num--;
@@ -1500,17 +1245,67 @@ fb_dirview_set_history(Futaba *futaba,
 	else if (num > 1)
 		msg_num = g_strdup_printf(_(" %d items exist"), num);
 
-	utf8 = str_local_to_utf8(abs_path);
+	utf8 = str_local_to_utf8(path);
 	msg_path = g_strdup_printf(_("\"%s\" open,"), utf8);
 	g_free(utf8);
 
 	message = g_strconcat(msg_path, msg_num, NULL);
 	fb_statusbar_swap(message);
 
-	g_free(abs_path);
 	g_free(msg_num);
 	g_free(msg_path);
 	g_free(message);
+}
+
+static gboolean
+fb_dirview_set_path(Futaba *futaba,
+		    gchar *path)
+{
+	GtkTreeModel *filter;
+	GtkAdjustment *adjust;
+
+	if (! path || ! *path) return FALSE;
+	if (!futaba->treeview) return FALSE;
+	if (!futaba->liststore) return FALSE;
+
+	if (!path_is_dir(path)) {
+		fb_statusbar_push(_("Directory don't exist."));
+
+		return FALSE;
+	}
+
+	/* set dirview */
+	if (!dirlist_set_dir(futaba->liststore, path, futaba->dot_filter, futaba->other_filter))
+		return FALSE;
+       
+	futaba->workdir = g_strdup(path);
+	futaba->writable = path_is_writable(path);
+	futaba->image_num = dirlist_get_count(futaba->liststore, IMAGE);
+
+	/* set row position */
+	adjust = gtk_tree_view_get_vadjustment(GTK_TREE_VIEW(futaba->treeview));
+	gtk_adjustment_set_value(adjust, 0.0);
+
+	return TRUE;
+}
+
+gboolean
+fb_dirview_set_history(Futaba *futaba,
+		       gchar *path)
+{
+	gchar *abs_path;
+
+	if (! path || ! *path) return FALSE; 
+
+	abs_path = get_absolute_path (path);
+	if (! abs_path) return FALSE;
+
+	if (! fb_dirview_set_path (futaba, abs_path))
+		return FALSE;
+
+	fb_dirview_show_message (futaba, abs_path);
+
+	g_free (abs_path);
 
 	return TRUE;
 }
@@ -1519,10 +1314,53 @@ gboolean
 fb_dirview_set_dir(Futaba *futaba,
 		   gchar *path)
 {
-	if (!fb_dirview_set_history(futaba, path))
+	gchar *abs_path;
+
+	if (! path || ! *path) return FALSE; 
+
+	abs_path = get_absolute_path (path);
+	if (! abs_path) return FALSE;
+
+	if (! fb_dirview_set_path (futaba, abs_path))
 		return FALSE;
 
-	fb_dir_history_add(futaba, path);
+	fb_dir_history_add (futaba, abs_path);
+
+	fb_dirview_show_message (futaba, abs_path);
+
+	g_free (abs_path);
+
+	return TRUE;
+}
+
+gboolean
+fb_dirview_set_archive(Futaba *futaba,
+		       gchar *path)
+{
+	gint len;
+	gchar *show_path;
+
+	if (! path || ! *path) return FALSE; 
+
+	if (! fb_dirview_set_path (futaba, path))
+		return FALSE;
+
+	fb_dir_history_add (futaba, path);
+
+	show_path = g_strconcat (g_get_home_dir (), FUTABA_EXTRACT_DIR, NULL);
+	len = strlen (show_path) - 1;
+	g_free (show_path);
+
+	show_path = g_strdup (path);
+	while ( len >= 0 ) {
+		show_path[len] = ' ';
+		len--;
+	}
+	show_path = g_strchug (show_path);
+
+	fb_dirview_show_message (futaba, show_path);
+
+	g_free (show_path);
 
 	return TRUE;
 }
@@ -1597,14 +1435,14 @@ fb_dirview_add_new_dir(Futaba *futaba)
 	if (!futaba->workdir) return;
 
 	path = g_strconcat(futaba->workdir, _("/New1"), NULL);
-	for (i = 2; g_file_test(path, G_FILE_TEST_EXISTS); i++) {
+	for (i = 2; path_is_exists(path); i++) {
 		g_free(path);
 		name = g_strdup_printf(_("/New%d"), i);
 		path = g_strconcat(futaba->workdir, name, NULL);
 		g_free(name);
 	}
 
-	if (!make_dir(path)) {
+	if (mkdir(path, S_IRWXU)) {
 		g_free(path);
 		return;
 	}
@@ -1720,8 +1558,8 @@ fb_dirview_get_pos(Futaba *futaba)
 }
 
 GList *
-fb_dirview_get_path(Futaba *futaba,
-		    FbFileType type)
+fb_dirview_get_path_list(Futaba *futaba,
+			 FbFileType type)
 {
 	GList *path_list;
 
@@ -1733,6 +1571,9 @@ fb_dirview_get_path(Futaba *futaba,
 	return path_list;
 }
 
+/* ɤ;Ϥ
+ * void fb_dirview_get_at_cursor(Futaba *futaba, ...)
+ */
 gboolean
 fb_dirview_get_readable_at_cursor(Futaba *futaba)
 {
@@ -1808,11 +1649,22 @@ fb_dirview_set_sort_mode(Futaba *futaba,
 	gtk_tree_sortable_set_sort_column_id(sortable, mode, order);
 }
 
+static void
+cb_archive_progress (FRArchive *archive,
+		     double fraction,
+		     gpointer data)
+{
+	FbProgDialog *dialog = data;
+
+	fb_progress_dialog_set_progress (dialog, CLAMP (fraction, 0, 1), NULL);
+}
+
 void
 fb_dirview_row_activated(Futaba *futaba)
 {
 	gboolean readable;
 	gint type;
+	gchar *path;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GtkTreeSelection *select;
@@ -1823,7 +1675,6 @@ fb_dirview_row_activated(Futaba *futaba)
 	if (!gtk_tree_selection_get_selected(select, &model, &iter)) return;
 
 	gtk_tree_model_get(model, &iter,
-			   COLUMN_TYPE, &type,
 			   COLUMN_READABLE, &readable,
 			   -1);
 
@@ -1832,20 +1683,66 @@ fb_dirview_row_activated(Futaba *futaba)
 		return;
 	}
 
+	gtk_tree_model_get(model, &iter,
+			   COLUMN_PATH, &path,
+			   COLUMN_TYPE, &type,
+			   -1);
+
 	switch (type) {
 	case DIRECTORY: {
-		gchar *path;
-
-		gtk_tree_model_get(model, &iter,
-				   COLUMN_PATH, &path,
-				   -1);
 		fb_dirview_set_dir(futaba, path);
 	}
 		break;
-	case IMAGE:
-		fb_imageviewer_start(futaba);
+	case IMAGE: {
+		gint num;
+		gchar *comp;
+		GList *path_list;
+
+		path_list = fb_dirview_get_path_list(futaba, IMAGE);
+		/* count position */
+		for (num = 0; comp = g_list_nth_data(path_list, num); num++) {
+			if (! g_ascii_strcasecmp(path, comp)) break;
+		}
+		fb_imageviewer_start(path_list, num);
+
+		path_list_free(path_list);
+		g_free(path);
+	}
 		break;
-	case ARCHIVE:
+	case ARCHIVE: {
+		FRArchive *archive;
+		GtkWidget *dialog;
+
+		dialog = fb_progress_dialog_new ("Preparing...");
+		gtk_widget_show (dialog);
+
+       		archive = fr_archive_new ();
+		g_signal_connect (G_OBJECT(archive), "done",
+				  G_CALLBACK(gtk_main_quit), NULL);
+		g_signal_connect(G_OBJECT (archive), "progress",
+				 G_CALLBACK (cb_archive_progress), dialog);
+		
+		if (fr_archive_load (archive, path, NULL)) {
+			gchar *extract_path;
+
+	     		gtk_main (); /* wait loading file list */
+
+			fb_progress_dialog_set_label (FB_PROGRESS_DIALOG (dialog), "Extracting...");
+		       	extract_path = g_strconcat (g_get_home_dir (), FUTABA_EXTRACT_DIR, path, NULL);
+			if (ensure_dir_exists (extract_path, S_IRWXU)) {
+
+				fr_archive_extract (archive, NULL, extract_path, NULL, 
+						    FALSE, TRUE, FALSE, NULL); 
+				gtk_main (); /* wait extracting archive */
+
+				fb_dirview_set_dir (futaba, extract_path);
+			}
+			g_free (extract_path);
+		}
+
+		g_object_unref (archive);
+		gtk_widget_destroy (dialog);
+	}
 		break;
 	case OTHER:
 	default:
