@@ -23,13 +23,12 @@
 #endif
 
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
 
 #include "futaba.h"
-#include "futaba_ui.h"
 #include "imageviewer.h"
 #include "canvas.h"
-#include "dirview.h"
+#include "viewer-action.h"
+#include "file-utils.h"
 
 #define FRAME 4
 #define CURSOR_TIMEOUT 2000
@@ -41,6 +40,9 @@ gint zoom_width;
 gint zoom_height;
 gint open_way = RIGHT;
 
+FbImageViewer *focus_viewer = NULL;
+GList *viewer_list = NULL;
+
 /*
  *-------------------------------------------------
  *       Utility
@@ -267,9 +269,7 @@ fb_zoom_window_destroy(FbImageViewer *viewer)
 static void cb_destroy(GtkObject *obj, gpointer data);
 static void cb_size_allocate(GtkWidget *wg, GtkAllocation *allocation, gpointer data);
 static gboolean cb_focus_in(GtkWidget *wg, GdkEventFocus *event, gpointer data);
-static gboolean cb_focus_out(GtkWidget *wg, GdkEventFocus *event, gpointer data);
 static gboolean cb_expose(GtkWidget *wg, GdkEventExpose *event, gpointer data);
-static gboolean cb_key_press(GtkWidget *wg, GdkEventKey *event, gpointer data);
 static gboolean cb_button_press(GtkWidget *wg, GdkEventButton *event, gpointer data);
 static gboolean cb_button_release(GtkWidget *wg, GdkEventButton *event, gpointer data);
 static gboolean cb_motion_notify(GtkWidget *wg, GdkEventMotion *event, gpointer data);
@@ -280,9 +280,14 @@ static void
 cb_destroy(GtkObject *obj,
 	   gpointer data)
 {
-  Futaba *futaba = data;
+	gint num;
+	FbImageViewer *viewer = data;
 
-  fb_imageviewer_end(futaba);
+	num = MAX(0, g_list_index(viewer_list, viewer) - 1);
+	viewer_list = g_list_remove(viewer_list, viewer);
+	fb_imageviewer_destroy(viewer);
+
+	focus_viewer = g_list_nth_data(viewer_list, num);
 }
 
 static gboolean
@@ -290,110 +295,13 @@ cb_focus_in(GtkWidget *wg,
 	    GdkEventFocus *event,
 	    gpointer data)
 {
-	gboolean flag;
-	GtkWidget *item1, *item2;
-	FbImageViewer *viewer;
-	Futaba *futaba = data;
+	FbImageViewer *viewer = data;
 
-	futaba->viewer = NULL;
-
-	viewer = g_object_get_data(G_OBJECT(wg), "Viewer");
-
-	if (viewer) viewer->focus = TRUE;
-
-	if (viewer->two_pages) {
-		item1 = gtk_ui_manager_get_widget(futaba->ui_manager,
-						  "/ViewerTwoPopup/TwoWindow");
-		item2 = gtk_ui_manager_get_widget(futaba->ui_manager,
-						  "/ViewerTwoPopup/FullScreen");
-	}
-	else {
-		item1 = gtk_ui_manager_get_widget(futaba->ui_manager,
-						  "/ViewerPopup/TwoWindow");
-		item2 = gtk_ui_manager_get_widget(futaba->ui_manager,
-						  "/ViewerPopup/FullScreen");
-	}
-
-	if (viewer->two_pages)
-		flag = TRUE;
-	else
-		flag = FALSE;
-
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item1), flag);
-
-	if (viewer->size > WINDOW_FULL)
-		flag = TRUE;
-	else
-		flag = FALSE;
-
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item2), flag);
-
-	futaba->viewer = viewer;
+	focus_viewer = viewer;
 
 	return TRUE;
 }
 
-static gboolean
-cb_focus_out(GtkWidget *wg,
-	     GdkEventFocus *event,
-	     gpointer data)
-{
-	Futaba *futaba = data;
-
-	futaba->viewer->focus = FALSE;
-
-	return TRUE;
-}
-
-static gboolean
-cb_key_press(GtkWidget *wg,
-	     GdkEventKey *event,
-	     gpointer data)
-{
-	Futaba *futaba = data;
-
-	/*   statusbar_pop(); */
-	g_print("key press: %x\n",event->keyval);
-
-	switch (event->keyval) {
-	case GDK_q:
-		g_signal_emit_by_name(G_OBJECT(wg), "destroy", futaba);
-		break;
-	case GDK_Home:
-		fb_imageviewer_first_page(futaba->viewer);
-		break;
-	case GDK_Right:
-	case GDK_n:
-		fb_imageviewer_next_pages(futaba->viewer, 1);
-		break;
-	case GDK_Down:
-		fb_imageviewer_next_pages(futaba->viewer, 10);
-		break;
-	case GDK_Page_Down:
-		fb_imageviewer_next_pages(futaba->viewer, 50);
-		break;
-	case GDK_Left:
-	case GDK_b:
-		fb_imageviewer_prev_pages(futaba->viewer, 1);
-		break;
-	case GDK_Up:
-		fb_imageviewer_prev_pages(futaba->viewer, 10);
-		break;
-	case GDK_Page_Up:
-		fb_imageviewer_prev_pages(futaba->viewer, 50);
-		break;
-	case GDK_End:
-	case GDK_e: /* temporary binding */
-		fb_imageviewer_last_page(futaba->viewer);
-		break;
-	case GDK_h:
-		gtk_window_iconify(GTK_WINDOW(wg));
-		break;
-	}
-
-	return FALSE;
-}
-
 static gboolean
 cb_button_press(GtkWidget *wg,
 		GdkEventButton *event,
@@ -401,8 +309,6 @@ cb_button_press(GtkWidget *wg,
 {
 	FbImageViewer *viewer = data;
 
-	/*   statusbar_pop(); */
-
 	viewer->pointer_last_x = event->x;
 	viewer->pointer_last_y = event->y;
 
@@ -430,9 +336,9 @@ cb_button_press(GtkWidget *wg,
 		GtkWidget *menu;
 
 		if (viewer->two_pages)
-			menu = viewer->twopopup;
+			menu = viewer->popup2;
 		else
-			menu = viewer->popup;
+			menu = viewer->popup1;
 
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
 			       event->button, event->time);
@@ -515,8 +421,6 @@ cb_scroll(GtkWidget *wg,
 {
 	FbImageViewer *viewer = data;
 
-	/*   statusbar_pop(); */
-
 	switch (event->scroll.direction) {
 	case 0:
 		fb_imageviewer_prev_pages(viewer, 1);
@@ -645,24 +549,24 @@ cb_size_allocate(GtkWidget *wg,
 }
 
 FbImageViewer *
-fb_imageviewer_new(Futaba *futaba)
+fb_imageviewer_new(GList *path_list)
 {
+	GtkUIManager *ui_manager;
 	GtkWidget *top, *frame, *draw;
 	FbImageViewer *viewer = g_new0(FbImageViewer, 1);
 
+	ui_manager = fb_viewer_popup_new(viewer);
+	if (!ui_manager) return NULL;
+
 	top = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_default_size(GTK_WINDOW(top), viewer_width, viewer_height);
 	gtk_window_set_title(GTK_WINDOW(top), "Viewer - Futaba");
-	gtk_window_add_accel_group(GTK_WINDOW(top), gtk_ui_manager_get_accel_group(futaba->ui_manager));
-	g_object_set_data(G_OBJECT(top), "Viewer", viewer);
+	gtk_window_add_accel_group(GTK_WINDOW(top), gtk_ui_manager_get_accel_group(ui_manager));
+	//	g_object_set_data(G_OBJECT(top), "Viewer", viewer);
 	g_signal_connect(G_OBJECT(top), "destroy",
-			 G_CALLBACK(cb_destroy), futaba);
+			 G_CALLBACK(cb_destroy), viewer);
 	g_signal_connect(G_OBJECT(top), "focus-in-event",
-			 G_CALLBACK(cb_focus_in), futaba);
-	g_signal_connect(G_OBJECT(top), "focus-out-event",
-			 G_CALLBACK(cb_focus_out), futaba);
-	g_signal_connect(G_OBJECT(top), "key_press_event",
-			 G_CALLBACK(cb_key_press), futaba);
+			 G_CALLBACK(cb_focus_in), viewer);
 	g_signal_connect(G_OBJECT(top), "size_allocate",
 			 G_CALLBACK(cb_size_allocate), viewer);
 	g_signal_connect(G_OBJECT(top), "leave_notify_event",
@@ -696,14 +600,12 @@ fb_imageviewer_new(Futaba *futaba)
 	viewer->top = top;
 	viewer->frame = frame;
 	viewer->draw = draw;
-	viewer->popup = gtk_ui_manager_get_widget(futaba->ui_manager,
-						  "/ViewerPopup");
-	viewer->twopopup = gtk_ui_manager_get_widget(futaba->ui_manager,
-						     "/ViewerTwoPopup");
+	viewer->popup1 = gtk_ui_manager_get_widget(ui_manager, "/Popup1");
+	viewer->popup2 = gtk_ui_manager_get_widget(ui_manager, "/Popup2");
 
 	viewer->first = NULL;
 	viewer->second = NULL;
-	viewer->path_list = fb_dirview_get_path(futaba, IMAGE);
+	viewer->path_list = path_list_dup(path_list);
 	viewer->position = 0;
 
 	viewer->width = viewer_width - FRAME;
@@ -718,43 +620,49 @@ fb_imageviewer_new(Futaba *futaba)
 	viewer->two_pages = FALSE;
 	viewer->open_way = open_way;
 	viewer->forward = TRUE;
-	viewer->focus = TRUE;
 	viewer->zooming = FALSE;
 
 	return viewer;
 }
 
-void
-fb_imageviewer_start(Futaba *futaba)
+FbImageViewer *
+fb_imageviewer_new_at_page(GList *path_list,
+			   gint num)
 {
-	gint i;
-	gchar *path, *comp;
-	FbFileType type;
+	gchar *path;
 	FbImageCanvas *fic;
 	FbImageViewer *viewer;
 
-	viewer = fb_imageviewer_new(futaba);
-	futaba->viewer = viewer;
-	futaba->viewer_list = g_list_append(futaba->viewer_list, viewer);
+	if (! path_list) return NULL;
 
-	/* check position of image */
-	type = fb_dirview_get_type_at_cursor(futaba);
-	if (type == IMAGE) {
-		path = fb_dirview_get_path_at_cursor(futaba);
-		for (i = 0; comp = g_list_nth_data(viewer->path_list, i); i++) {
-			if (!g_ascii_strcasecmp(path, comp)) break;
-		}
-	}
-	else {
-		i = 0;
-		path = g_list_nth_data(viewer->path_list, i);
-	}
-	viewer->position = i;
+	viewer = fb_imageviewer_new(path_list);
+	if (! viewer) return NULL;
+
+	viewer->position = MAX( 0, num);
+	path = g_list_nth_data(viewer->path_list, viewer->position);
+	if (! path) return viewer;
 
 	fic = fb_imagecanvas_new_from_file(viewer->draw, path, 0, 0, viewer->width, viewer->height);
 	fb_imagecanvas_draw(fic, fic->canvas);
   
 	viewer->first = fic;
+
+	return viewer;
+}
+
+void
+fb_imageviewer_start(GList *path_list,
+		     gint n)
+{
+	FbImageViewer *viewer;
+
+
+	if (! path_list) return;
+
+	viewer = fb_imageviewer_new_at_page(path_list, n);
+
+	focus_viewer = viewer;
+	viewer_list = g_list_append(viewer_list, viewer);
 }
 
 void
@@ -952,9 +860,7 @@ fb_imageviewer_destroy(FbImageViewer *viewer)
 	if (viewer->second)
 		fb_imagecanvas_free(viewer->second);
 
-	for (; viewer->path_list; viewer->path_list = viewer->path_list->next)
-		g_free(viewer->path_list->data);
-	g_list_free(viewer->path_list);
+	path_list_free(viewer->path_list);
 
 	if (GTK_IS_WIDGET(viewer->draw)) gtk_widget_destroy(viewer->draw);
 
@@ -966,20 +872,5 @@ fb_imageviewer_destroy(FbImageViewer *viewer)
 
 	g_free(viewer);
 
-	g_print("Image Viewer destroy\n");
-}
-
-void
-fb_imageviewer_end(Futaba *futaba)
-{
-	gint i;
-
-	if (!futaba->viewer) return;
-
-	i = MAX(0, g_list_index(futaba->viewer_list, futaba->viewer) - 1);
-
-	futaba->viewer_list = g_list_remove(futaba->viewer_list, futaba->viewer);
-	fb_imageviewer_destroy(futaba->viewer);
-
-	futaba->viewer = g_list_nth_data(futaba->viewer_list, i);
+	g_print("End Viewer\n");
 }
