@@ -22,6 +22,7 @@
 #define __THUMBNAIL_H__
 
 typedef struct _FbThumbnail FbThumbnail;
+typedef struct _FbThumbWin FbThumbWin;
 
 struct _FbThumbnail
 {
@@ -50,7 +51,9 @@ struct _FbThumbWin
 	gint height;
 };
 
-void fb_thumbwin_start(Futaba *futaba);
-void fb_thumbwin_end(Futaba *futaba);
+extern FbThumbWin *focus_thumbwin;
+
+void fb_thumbwin_start(GList *path_list);
+void fb_thumbwin_end(FbThumbWin *thumbwin);
 
 #endif /* __THUMBNAIL_H__ */
